import { BehaviorSubject } from 'rxjs';
import type { Metadata, SchemaResponse } from '../metadata';
import { ConnectConfig, ConnectionStatus } from './connect.types';
import { CommitParams, Message, RequestParams } from './message';
import type { SocketObservable } from './socket';
/**
 * The default configuration for connecting to a WS service.
 * @public
 */
export declare const defaultConnectConfig: ConnectConfig;
/**
 * Represents a connection and its capabilities on a remote server
 * Provides methods to send and receive messages, as well as metadata and schema information.
 * @public
 */
export interface Connect {
    /**
     * Indicates whether the connection is currently established.
     * @public
     */
    isConnected: boolean;
    /**
     * The current status of the connection.
     * @remarks Possible values are 'connected', 'disconnected', and 'busy'.
     * @public
     */
    status: ConnectionStatus;
    /**
     * A behavior subject holding the current connection status.
     * @internal
     */
    _status: BehaviorSubject<ConnectionStatus>;
    /**
     * A behavior subject holding the current host URL.
     * @internal
     */
    _hostUrl: BehaviorSubject<String>;
    /**
     * Sends a message to the server and returns a promise that resolves with the server's response.
     * @internal
     * @param message - The message to send.
     * @typeParam T - The type of the Message's DETAILS payload.
     * @param needsHandling - Whether the response needs to be handled manually. Default: false.
     * @returns A promise that resolves with the server's response.
     */
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    /**
     * Returns whether the connection is in HTTP mode.
     * @internal
     * @returns True if the connection is in HTTP mode; false otherwise.
     */
    httpMode(): boolean;
    /**
     * Connects to a server at the specified host.
     * @public
     * @param host - The host to connect to.
     * @returns A promise that resolves with a boolean indicating whether the connection was successful.
     */
    connect(host: string): Promise<boolean>;
    /**
     * Disconnects from the server.
     * @public
     */
    disconnect(): void;
    /**
     * Sets the validity of the current session.
     * Use in Auth scenarios where session expires or renews, but the connection is still active.
     * @internal
     * @param valid - Whether the session is valid.
     */
    setValidSession(valid: boolean): void;
    /**
     * Retrieves a snapshot of data for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the snapshot for.
     * @param params - Additional parameters to pass to the server.
     * @returns A promise that resolves with the snapshot data.
     */
    snapshot(resourceName: string, params?: any): Promise<Message>;
    /**
     * Starts listening for updates on the specified resource.
     * @public
     * @param resourceName - The name of the resource to listen for updates on.
     * @param onMessage - A callback function to handle received messages.
     * @param onError - A callback function to handle errors.
     * @param params - Additional parameters to pass to the server.
     * @returns An observable that emits messages received from the server.
     */
    stream(resourceName: string, onMessage: Function, onError: Function, params?: any): SocketObservable<Message>;
    /**
     * Requests additional rows of data for the specified source reference.
     * @public
     * @param sourceRef - The source reference to request more rows for.
     * @returns A promise that resolves with the additional rows of data.
     */
    getMoreRows(sourceRef: string): Promise<Message>;
    /**
     * Logs off from the specified data source.
     * @public
     * @param sourceRef - The reference of the data source to log off from.
     * @returns A promise that resolves with a message indicating success or failure.
     */
    dataLogoff(sourceRef: string): Promise<Message>;
    /**
     * Retrieves the metadata for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the metadata for.
     * @returns A promise that resolves with the metadata.
     */
    getMetadata(resourceName: string): Promise<Metadata>;
    /**
     * Retrieves the JSON schema for the specified resource.
     * @public
     * @param resourceName - The name of the resource to retrieve the schema for.
     */
    getJSONSchema(resourceName: string): Promise<SchemaResponse>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    request(resourceName: string, params?: RequestParams): Promise<Message>;
    /**
     * Sends a request to the server to retrieve data or perform an action.
     * @public
     * @param resourceName - The name of the resource to request.
     * @param params - An object containing optional parameters to include in the request.
     * @returns A promise that resolves to a Message containing the server's response.
     */
    commitEvent(eventName: string, params?: CommitParams): Promise<Message>;
}
/**
 * The DI token for the Connect interface (WS-only).
 * @internal
 */
export declare const WSConnect: import("@microsoft/fast-foundation").InterfaceSymbol<Connect>;
/**
 * The DI token for the Connect interface (HTTP | WS).
 * @internal
 */
export declare const Connect: import("@microsoft/fast-foundation").InterfaceSymbol<Connect>;
//# sourceMappingURL=connect.d.ts.map